@@ -19,10 +19,19 @@ class VariableReference(NamedTuple):
     context: "System"
     mapping: "Union[Mapping[str, Any], ExtensiblePort]"
     key: str
+
+    @property
+    def value(self) -> Any:
+        """Any: variable reference value"""
+        return self.mapping[self.key]
+
+    @value.setter
+    def value(self, value: Any) -> None:
+        self.mapping[self.key] = value
     
     def __repr__(self) -> str:
         name = self.context.name
         mapping_name = getattr(self.mapping, "name", "")
         if mapping_name:
-            name = ".".join((name, mapping_name))
-        return f'<VariableReference ({name}, {self.key})>'
+            name = f"{name}.{mapping_name}"
+        return f"<VariableReference ({name}, {self.key})>"
