@@ -137,7 +137,6 @@ def test_RungeKutta_twoTanks(two_tank_case, two_tank_solution, dt):
     )
     
     recorder = driver.add_recorder(recorders.DataFrameRecorder(includes='tank?.height'), period=0.1)
-    # recorder = driver.add_recorder(recorders.DSVRecorder('twoTanks_RK{}.csv'.format(order), includes=['tank?.height', ]), period=0.1)
     assert driver.recording_period == 0.1
 
     system.run_drivers()
@@ -146,15 +145,14 @@ def test_RungeKutta_twoTanks(two_tank_case, two_tank_solution, dt):
     assert system.tank2.height > 1
     assert system.tank1.height == pytest.approx(system.tank2.height, rel=1e-3)
 
-    assert len(recorder.data) == 51
-    heights = recorder.data.values[:, -2:]
+    data = recorder.export_data()
+    assert len(data) == 51
     solution = two_tank_solution(system, init)
     assert solution.characteristic_time == pytest.approx(0.5766040318109212)
     assert driver.dt < solution.characteristic_time
-    time = np.array(recorder.data['Reference'], dtype=float)
+    time = np.array(data['Reference'], dtype=float)
     error = 0
-    for i, h1 in enumerate(heights[1:, 0]):
-        t = time[i + 1]
+    for t, h1 in zip(time, data['tank1.height']):
         exact = solution(t)
         error = max(error, abs(h1 - exact[0]))
     # Test that maximum error ~ dt^2
@@ -195,7 +193,7 @@ def test_RungeKutta_scalar_ode(scalar_ode_case, case, settings, tol):
     )
     ode.run_drivers()
     # Retrieve recorded data and check accuracy
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
     result = np.asarray(data['f'], dtype=float)
     solution = case['solution']
@@ -232,7 +230,7 @@ def test_RungeKutta_scalar_ode_limited_dt(settings, expected):
     )
     ode.run_drivers()
     # Retrieve recorded data and check accuracy
-    data = recorder.data
+    data = recorder.export_data()
     times = np.asarray(data['Reference'], dtype=float)
     ys = np.asarray(data['y'], dtype=float)
     zs = np.asarray(data['z'], dtype=float)
@@ -265,7 +263,7 @@ def test_RungeKutta_vector_ode(vector_ode_case, settings, tol):
 
     ode.run_drivers()
     # Retrieve recorded data and check accuracy
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
     solution = lambda t, x0: np.array([t**2, np.log(1 + t), 1 - np.exp(-t)]) + x0
     result = np.asarray([value for value in data['v']])
@@ -284,7 +282,6 @@ def test_RungeKutta_point_mass(point_mass_case, point_mass_solution, order, dt,
     settings = dict(order=order, time_interval=(0, 2), dt=dt)
     system, driver = point_mass_case(RungeKutta, **settings)
 
-    # recorder = driver.add_recorder(recorders.DSVRecorder('pointMass_RK{}.csv'.format(driver.order), includes=['x', 'v', 'a']), period=0.1)
     recorder = driver.add_recorder(recorders.DataFrameRecorder(includes=['x', 'v', 'a']), period=0.1)
 
     x0 = [-1., 0., 10]
@@ -296,15 +293,13 @@ def test_RungeKutta_point_mass(point_mass_case, point_mass_solution, order, dt,
 
     system.run_drivers()
 
-    data = recorder.data
+    data = recorder.export_data()
     time = np.array(data['Reference'], dtype=float)
-    traj = np.asarray([value for value in data.values[:, -1]])
     solution = point_mass_solution(system, v0, x0)
     error = np.zeros(3)
-    for i, t in enumerate(time):
-        x = traj[i]
+    for t, x in zip(time, data['x']):
         error = np.maximum(error, rel_error(x, solution.x(t)))
-    context = 'dt = {}, order = {}'.format(driver.dt, driver.order)
+    context = f"dt = {driver.dt}, order = {driver.order}"
     assert error.max() < tol, context
 
 
@@ -388,7 +383,6 @@ def test_RungeKutta_pointMassWithPorts(pointMassWithPorts_case, point_mass_solut
     system, driver = pointMassWithPorts_case(RungeKutta, **settings)
 
     includes = ['pos*.x', 'kin*.v', 'a']
-    # recorder = driver.add_recorder(recorders.DSVRecorder('pointMassWithPorts_RK{}.csv'.format(driver.order), includes=includes), period=0.1)
     recorder = driver.add_recorder(recorders.DataFrameRecorder(includes=includes), period=0.1)
 
     x0 = [-1., 0., 10]
@@ -400,15 +394,14 @@ def test_RungeKutta_pointMassWithPorts(pointMassWithPorts_case, point_mass_solut
 
     system.run_drivers()
 
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
-    traj = np.asarray([value for value in data.values[:, -1]])
+    traj = np.asarray(data['position.x'])
     solution = point_mass_solution(system, v0, x0)
     error = np.zeros(3)
-    for i, t in enumerate(time):
-        x = traj[i]
+    for t, x in zip(time, traj):
         error = np.maximum(error, rel_error(x, solution.x(t)))
-    context = 'dt = {}, order = {}'.format(driver.dt, driver.order)
+    context = f"dt = {driver.dt}, order = {driver.order}"
     assert error.max() < tol, context
 
 
@@ -434,7 +427,6 @@ def test_RungeKutta_pointMassWithPorts_pulling(point_mass_solution, order, dt, t
     system, driver = make_case(RungeKutta, **settings)
 
     includes = ['*.x', '*.v', '*.a']
-    # recorder = driver.add_recorder(recorders.DSVRecorder('pointMassWithPorts_RK{}.csv'.format(driver.order), includes=includes), period=0.1)
     recorder = driver.add_recorder(recorders.DataFrameRecorder(includes=includes), period=0.1)
 
     x0 = [-1., 0., 10]
@@ -446,15 +438,14 @@ def test_RungeKutta_pointMassWithPorts_pulling(point_mass_solution, order, dt, t
 
     system.run_drivers()
 
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
-    traj = np.asarray([value for value in data.values[:, -1]])
+    traj = np.asarray(data['pos.x'])
     solution = point_mass_solution(system.point, v0, x0)
     error = np.zeros(3)
-    for i, t in enumerate(time):
-        x = traj[i]
+    for t, x in zip(time, traj):
         error = np.maximum(error, rel_error(x, solution.x(t)))
-    context = 'dt = {}, order = {}'.format(driver.dt, driver.order)
+    context = f"dt = {driver.dt}, order = {driver.order}"
     assert error.max() < tol, context
 
 
@@ -465,7 +456,7 @@ def test_RungeKutta_pointMassWithPorts_pulling(point_mass_solution, order, dt, t
 def test_RungeKutta_rate_singleTimeStep(rate_case_1, dt, tol):
     settings = dict(order=2, time_interval=(0, dt), dt=dt)
     system, driver = rate_case_1(RungeKutta, **settings)
-    context = 'dt = {}, order = {}'.format(driver.dt, driver.order)
+    context = f"dt = {driver.dt}, order = {driver.order}"
     assert driver.dt == driver.time_interval[1], context
 
     driver.set_scenario(values={'k': 1.9, 'U': 'exp(k * t)'})
@@ -483,16 +474,15 @@ def test_RungeKutta_rate_singleTimeStep(rate_case_1, dt, tol):
 def test_RungeKutta_rate(rate_case_1, dt, tol):
     settings = dict(order=2, time_interval=(0, 1), dt=dt)
     system, driver = rate_case_1(RungeKutta, **settings)
-    context = 'dt = {}, order = {}'.format(driver.dt, driver.order)
+    context = f"dt = {driver.dt}, order = {driver.order}"
 
     driver.set_scenario(values={'k': 1.9, 'U': 'exp(k * t)'})
 
-    # recorder = driver.add_recorder(recorders.DSVRecorder('dUdt_RK{}.csv'.format(driver.order), includes=['U', 'dU_dt']), period=0.1)
     recorder = driver.add_recorder(recorders.DataFrameRecorder(includes=['dU_dt']), period=0.1)
 
     system.run_drivers()
 
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
     result = np.asarray(data['dU_dt'], dtype=float)
     solution = lambda t: system.k * np.exp(system.k * t)
@@ -534,8 +524,9 @@ def test_RungeKutta_oscillator(oscillator_case, oscillator_solution, parameters,
         assert driver.dt == settings['dt']
 
     solution = oscillator_solution(system, x0, v0)
-    time = np.array(recorder.data['Reference'], dtype=float)
-    x = np.asarray(recorder.data["x"])
+    data = recorder.export_data()
+    time = np.array(data['Reference'], dtype=float)
+    x = np.asarray(data['x'])
     error = 0
     for i, t in enumerate(time):
         error = max(error, abs(x[i] - solution.x(t)))
@@ -572,7 +563,7 @@ def test_RungeKutta_gaussian(gaussian_ode, parameters, settings, expected):
 
     ode.run_drivers()
     # Retrieve recorded data and check accuracy
-    data = recorder.data
+    data = recorder.export_data()
     time = np.asarray(data['Reference'], dtype=float)
     result = np.asarray(data['f'], dtype=float)
     t0 = driver.time_interval[0]
