@@ -3,12 +3,10 @@ Classes containing all `System` variables.
 """
 import array
 import logging
-import re
-import weakref
 from collections import OrderedDict
 from collections.abc import MutableSequence
-from typing import Any, Dict, Iterator, NoReturn, Optional, Tuple, Union
-
+from typing import Any, Dict, Iterator, Optional, Tuple, Union
+from types import MappingProxyType
 import numpy
 
 from cosapp.core.numerics.distributions.distribution import Distribution
@@ -36,7 +34,7 @@ class ExtensiblePort:
 
     _name_check = NameChecker()
 
-    def __init__(self, name: str, direction: PortType) -> NoReturn:
+    def __init__(self, name: str, direction: PortType) -> None:
         """`ExtensiblePort` constructor.
 
         Parameters
@@ -81,10 +79,30 @@ class ExtensiblePort:
 
         If the port has no owner, the port name is returned.
         """
-        if self._owner is None:
-            return self._name
-        else:
-            return f"{self._owner.name}.{self._name}"
+        owner = self._owner
+        return self._name if owner is None else f"{owner.name}.{self._name}"
+
+    def full_name(self, trim_root=False) -> str:
+        """Returns full name up to root owner.
+        
+        Parameters
+        ----------
+        trim_root : bool (optional, default False)
+            Exclude root owner name if True.
+
+        Returns
+        -------
+        str
+            The port full name
+        """
+        owner = self.owner
+        path = []
+        if owner is not None:
+            path = owner.path_namelist()
+            if trim_root:
+                path = path[1:]
+        path.append(self.name)
+        return ".".join(path)
 
     @property
     def direction(self) -> PortType:
@@ -147,7 +165,7 @@ class ExtensiblePort:
         desc: str = "",
         distribution: Optional[Distribution] = None,
         scope: Scope = Scope.PRIVATE,
-    ) -> NoReturn:
+    ) -> None:
         """Add a variable to the port.
 
         The `valid_range` defines the range of value for which a model is known to behave correctly.
@@ -211,7 +229,7 @@ class ExtensiblePort:
         setattr(self, name, value)
 
     # TODO unused and should be removed -> to dangerous for consistency
-    def remove_variable(self, name: str) -> NoReturn:
+    def remove_variable(self, name: str) -> None:
         """Remove the variable from the port.
 
         Parameters
@@ -241,7 +259,7 @@ class ExtensiblePort:
         if key in self._variables and self._owner:
             self._owner.set_dirty(self.direction)
 
-    def validate(self, key: str, value: Any) -> NoReturn:
+    def validate(self, key: str, value: Any) -> None:
         """Check if a variable is in the scope of the user and the type is valid.
         
         Parameters
@@ -297,7 +315,7 @@ class ExtensiblePort:
     __setattr__ = __set_variable
 
     @staticmethod
-    def set_type_checking(activate: bool) -> NoReturn:
+    def set_type_checking(activate: bool) -> None:
         """(Un)set type checking when affecting port variables.
 
         By default type checking is activated.
@@ -316,12 +334,12 @@ class ExtensiblePort:
         return item in self._variables
 
     def __getitem__(self, item: str) -> Any:
-        if item in self._variables:
+        try:
             return getattr(self, item)
-        else:
-            raise KeyError(f"Variable {item} does not exist in Port {self}.")
+        except AttributeError:
+            raise KeyError(f"Variable or property {item} does not exist in Port {self}.")
 
-    def __setitem__(self, key: str, value: Any) -> NoReturn:
+    def __setitem__(self, key: str, value: Any) -> None:
         if key in self._variables:
             setattr(self, key, value)
         else:
@@ -340,12 +358,12 @@ class ExtensiblePort:
         return self.__clearance
 
     @scope_clearance.setter
-    def scope_clearance(self, user_scope: Scope) -> NoReturn:
+    def scope_clearance(self, user_scope: Scope) -> None:
         check_arg(user_scope, "scope_clearance", Scope)
         self.__clearance = user_scope
         self.__update_filter()
 
-    def __update_filter(self) -> NoReturn:
+    def __update_filter(self) -> None:
         """Update the implementation of method `out_of_scope`
         according to port's clearance level and owner."""
         if self._owner is None:
@@ -404,13 +422,11 @@ class ExtensiblePort:
 
         Returns
         -------
-        weakref.WeakValueDictionary[str, cosapp.ports.variable.Variable] or cosapp.ports.variable.Variable
-            The variable looked for or all port variables
+        types.MappingProxyType[str, cosapp.ports.variable.Variable] or cosapp.ports.variable.Variable
+            The sought variable, or a read-only view on all port variables.
         """
         if name is None:
-            # Proxy through a weakref value dictionary to avoid changes in the reference
-            # and ensuring minimal memory foot print
-            return weakref.WeakValueDictionary(self._variables)
+            return MappingProxyType(self._variables)
         else:
             return self._variables[name]
 
@@ -504,7 +520,7 @@ class ExtensiblePort:
 
         return new_port
 
-    def morph(self, port: "ExtensiblePort") -> NoReturn:
+    def morph(self, port: "ExtensiblePort") -> None:
         """Morph the provided port into this port.
 
         Morphing a port is useful when converting a `System` in something equivalent. The morphing
@@ -537,8 +553,13 @@ class ExtensiblePort:
             if variable not in self:
                 port.remove_variable(variable)
 
-    def to_dict(self) -> Dict[str, Union[str, Tuple[Dict[str, str], str]]]:
+    def to_dict(self, with_def: bool = False) -> Dict[str, Union[str, Tuple[Dict[str, str], str]]]:
         """Convert this port in a dictionary.
+   
+        Parameters
+        ----------
+        with_def : bool
+            Flag to export also output ports and its class name (default: False).
 
         Returns
         -------
@@ -548,12 +569,24 @@ class ExtensiblePort:
         # TODO this is uncomplete as validation ranges and distribution could be changed
         new_dict = dict()
 
-        if self.direction == PortType.IN:
-            for variable in self:
-                fullname = ".".join((self.name, variable))
-                value = getattr(self, variable)
-                new_dict[fullname] = value
+        if with_def:
+            tmp =  dict()
+            if self.name not in ["inwards", "outwards"]:
+                tmp["__class__"] = self.__class__.__qualname__
+                for variable in self:
+                    tmp[variable] = getattr(self, variable)
+            else:
+                for v_name, variable in self._variables.items():
+                    tmp[v_name]= variable.to_dict() 
 
+            new_dict[self.name] = tmp
+        
+        else:
+            if self.direction == PortType.IN:
+                for variable in self:
+                    fullname = f"{self.name}.{variable}"
+                    new_dict[fullname] = getattr(self, variable)
+        
         return new_dict
 
 
@@ -614,7 +647,7 @@ class Port(ExtensiblePort):
 
     def __init__(
         self, name: str, direction: PortType, variables: Optional[Dict[str, Any]] = None
-    ) -> NoReturn:
+    ) -> None:
         """`Port` constructor.
 
         An optional dictionary may be specified to overwrite variable value and some of their
@@ -662,7 +695,7 @@ class Port(ExtensiblePort):
 
         self._locked = True
 
-    def setup(self) -> NoReturn:
+    def setup(self) -> None:
         """`Port` variables are defined in this function by calling `add_variable`.
 
         This function allows to populate a customized `Port` class. The `add_variable`
@@ -694,7 +727,7 @@ class Port(ExtensiblePort):
         desc: str = "",
         distribution: Optional[Distribution] = None,
         scope=Scope.PUBLIC,
-    ) -> NoReturn:
+    ) -> None:
         """Add a variable to the port.
 
         The `valid_range` defines the range of value for which a model is known to behave correctly.
@@ -748,7 +781,7 @@ class Port(ExtensiblePort):
         )
 
     # TODO remove ?
-    def remove_variable(self, name: str) -> NoReturn:
+    def remove_variable(self, name: str) -> None:
         """Remove the variable from the port.
 
         Parameters
